/* bb910_driver.c */
/* drvBb910.c,v 1.2 2004/02/04 18:53:20 mrk Exp */
/***********************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of
* California, as Operator of Los Alamos National Laboratory.
* burrBrown is distributed subject to a Software License Agreement
* found in file LICENSE that is included with this distribution.
***********************************************************************/
/*
 * subroutines that are used to interface to the binary input cards
 *
 * 	Author:      Bob Dalesio
 * 	Date:        6-13-88
 *
 * Modification Log:
 * -----------------
 * .01	02-09-89	lrd	moved I/O addresses to module_types.h
 * .02	11-20-89	joh 	added call to at5vxi driver
 * .03  09-11-91	 bg	added bb910_io_report
 * .04  10-31-91	 bg	broke bb910 code out of bi_driver.c
 * .05  02-04-92	 bg	added the argument level to 
 * 				bb910_io_report() and gave it the ability
 *                              to read raw values from card if level > 0 
 * .06	08-10-92	joh	made the number of cards runtime
 * .07	08-25-92	mrk	made masks a macro
 */

/*
 * Code Portions:
 *
 * bi_driver_init  Finds and initializes all binary input cards present
 * bi_driver       Interfaces to the binary input cards present
 */


#include <vxWorks.h>
#include <stdlib.h>
#include <stdio.h>
#include <vxLib.h>
#include <sysLib.h>
#include <vme.h>
#include <epicsExport.h>

#include "drvSup.h"

#include "drvBb910.h"

static short bi_num_cards[] = {4};
static unsigned short bi_addrs[] = {0xb800};
static short bi_num_channels[] = {32};

/* Number of columns used in io_report. */
#define IOR_MAX_COLS 4

struct {
        long    number;
        DRVSUPFUN       report;
        DRVSUPFUN       init;
} drvBb910={
        2,
        bb910_io_report,
        bb910_driver_init};
epicsExportAddress(drvet,drvBb910);


static struct {char ** ppc; char*pc;} RCSID = {&(RCSID.pc),
    "$Id"};

#define MAX_BB_BI_CARDS	(bi_num_cards[BB910]) 

/* Burr-Brown 910 binary input memory structure */
/* Note: the high and low order words are switched from the io card */
struct bi_bb910{
        unsigned short  csr;            /* control status register */
        unsigned short  low_value;      /* low order 16 bits value */
        unsigned short  high_value;     /* high order 16 bits value */
        char    end_pad[0x100-6];       /* pad until next card */
};

/* pointers to the binary input cards */
struct bi_bb910 **pbi_bb910s;      /* Burr-Brown 910s */

static char *bb910_shortaddr;


/*
 * BI_DRIVER_INIT
 *
 * intialization for the binary input cards
 */

long bb910_driver_init(){
	int bimode;
        int status;
        short  i;
        struct bi_bb910        *pbi_bb910;

	pbi_bb910s = (struct bi_bb910 **) 
		calloc(MAX_BB_BI_CARDS, sizeof(*pbi_bb910s));
	if(!pbi_bb910s){
		return ERROR;
	}

        /* intialize the Burr-Brown 910 binary input cards */
        /* base address of the burr-brown 910 binary input cards */

        status=sysBusToLocalAdrs(VME_AM_SUP_SHORT_IO,(char *)(long)bi_addrs[BB910],&bb910_shortaddr);
        if (status != OK){ 
           printf("Addressing error in bb910 driver\n");
           return ERROR;
        }
        pbi_bb910 = (struct bi_bb910 *)bb910_shortaddr;

        /* determine which cards are present */
        for (i = 0; i < bi_num_cards[BB910]; i++,pbi_bb910++){
            if (vxMemProbe((char *)pbi_bb910,READ,sizeof(short),(char *)&bimode) == OK){
                pbi_bb910s[i] = pbi_bb910;
            }
            else {
                pbi_bb910s[i] = 0;
            }
        }
        
        return (0);

}


long bb910_driver(short card, u_long mask, u_long *prval)
{
	u_long 	work;

	if (!pbi_bb910s[card])
       		return (-1);

        /* read */

        work = (pbi_bb910s[card]->high_value << 16)    /* high */
        	+ pbi_bb910s[card]->low_value;             /* low */
	/* apply mask */
	*prval = work & mask;

	return (0);             
   }

#define masks(K) ((1<<K))
long bb910_io_report(int level)
 { 
   short i,j,k,l,m,num_chans;
   u_long jval,kval,lval,mval;

   for (i = 0; i < bi_num_cards[BB910]; i++){
	if (pbi_bb910s[i]){
           printf("BI: BB910:      card %d\n",i);
           if (level > 0){
               num_chans = bi_num_channels[BB910];
               for(j=0,k=1,l=2,m=3;
	           j < num_chans && k < num_chans && l < num_chans && m < num_chans;
                   j+=IOR_MAX_COLS,k+= IOR_MAX_COLS,l+= IOR_MAX_COLS,m += IOR_MAX_COLS){
        	if(j < num_chans){
                        bb910_driver(i,masks(j),&jval);
                 	if (jval != 0) 
                  		 jval = 1;
                         printf("Chan %d = %lx\t ",j,jval);
                }  
         	if(k < num_chans){
                        bb910_driver(i,masks(k),&kval);
                        if (kval != 0) 
                        	kval = 1;
                        printf("Chan %d = %lx\t ",k,kval);
                }
                if(l < num_chans){
                        bb910_driver(i,masks(l),&lval);
                	if (lval != 0) 
                        	lval = 1;
                	printf("Chan %d = %lx \t",l,lval);
                 }
                  if(m < num_chans){
                        bb910_driver(i,masks(m),&mval);
                 	if (mval != 0) 
                        	mval = 1;
                 	printf("Chan %d = %lx \n",m,mval);
                   }
             }
           }
        }
   }
   return 0;
 }

int bbMpf910Config(unsigned int ncards, unsigned int base,
    unsigned int nchannels)
{
    bi_num_cards[BB910] = ncards;
    bi_addrs[BB910] = base;
    bi_num_channels[BB910] = nchannels;
    return 0;
}
